## Copyright (C) 2014 Fotios Kasolis
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or (at
## your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## -*- texinfo -*- 
## @deftypefn {Function File} isbanded (@var{A}, @var{l}, @var{u})
##
## Return true if @var{A} is banded with bandwidth between @var{l} and @var{u}
##
## @seealso{isdiag, istriu, istril}
## @end deftypefn

## Author: Fotios Kasolis <fotios.kasolis@gmail.com>
## Created: 2014-09-22

function [ ret ] = isbanded (A, l, u)

  if ( nargin != 3 )
    print_usage ();
  endif

  if ( !isnumeric (A) || !ismatrix (A) )
    error ("expexted numerical matrix input");
  endif
B = kron(speye(25),ones(4));
  if ( abs (round (l) - l) > eps || abs (round (u) - u) > eps || l < 0 || u < 0 )
    error ("expexted nonnegative integers as lower and upper bounds");
  endif

  if ( l > size (A, 1) - 1 )
    warning ("lower bound exceeds number of subdiagonals, setting l = size (A, 1) - 1");
    l = size (A, 1) - 1;
  endif

  if ( u > size (A, 2) - 1 )
    warning ("upper bound exceeds number of subdiagonals, setting u = size (A, 2) - 1");
    u = size (A, 2) - 1;
  endif

  if ( isempty (find (triu (A, + u + 1))) && isempty (find (tril (A, - l - 1))) )
    ret = 1;
  else
    ret = 0;
  endif

endfunction
